@@ -1,5 +1,5 @@
 /* seq - print sequence of numbers to standard output.
-   Copyright (C) 1994-2004 Free Software Foundation, Inc.
+   Copyright (C) 1994-2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /* Written by Ulrich Drepper.  */
 
@@ -26,6 +26,7 @@
 #include "system.h"
 #include "c-strtod.h"
 #include "error.h"
+#include "quote.h"
 #include "xstrtol.h"
 #include "xstrtod.h"
 
@@ -34,8 +35,8 @@
 
 #define AUTHORS "Ulrich Drepper"
 
-/* If nonzero print all number with equal width.  */
-static int equal_width;
+/* If true print all number with equal width.  */
+static bool equal_width;
 
 /* The name that this program was run with.  */
 char *program_name;
@@ -48,9 +49,8 @@ static char *separator;
 /* FIXME: make this an option.  */
 static char *terminator = "\n";
 
-/* The representation of the decimal point in the current locale.
-   Always "." if the localeconv function is not supported.  */
-static char *decimal_point = ".";
+/* The representation of the decimal point in the current locale.  */
+static char decimal_point;
 
 /* The starting number.  */
 static double first;
@@ -116,7 +116,7 @@ scan_double_arg (const char *arg)
 {
   double ret_val;
 
-  if (xstrtod (arg, NULL, &ret_val, c_strtod))
+  if (! xstrtod (arg, NULL, &ret_val, c_strtod))
     {
       error (0, 0, _("invalid floating point argument: %s"), arg);
       usage (EXIT_FAILURE);
@@ -125,10 +125,10 @@ scan_double_arg (const char *arg)
   return ret_val;
 }
 
-/* Check whether the format string is valid for a single `double'
-   argument.  Return 0 if not, 1 if correct. */
+/* Return true if the format string is valid for a single `double'
+   argument.  */
 
-static int
+static bool
 valid_format (const char *fmt)
 {
   while (*fmt != '\0')
@@ -143,7 +143,7 @@ valid_format (const char *fmt)
       fmt++;
     }
   if (*fmt == '\0')
-    return 0;
+    return false;
 
   fmt += strspn (fmt, "-+#0 '");
   if (ISDIGIT (*fmt) || *fmt == '.')
@@ -158,7 +158,7 @@ valid_format (const char *fmt)
     }
 
   if (!(*fmt == 'e' || *fmt == 'f' || *fmt == 'g'))
-    return 0;
+    return false;
 
   fmt++;
   while (*fmt != '\0')
@@ -167,13 +167,13 @@ valid_format (const char *fmt)
 	{
 	  fmt++;
 	  if (*fmt != '%')
-	    return 0;
+	    return false;
 	}
 
       fmt++;
     }
 
-  return 1;
+  return true;
 }
 
 /* Actually print the sequence of numbers in the specified range, with the
@@ -203,7 +203,7 @@ print_numbers (const char *fmt)
    will format to strings of the same width.  */
 
 static char *
-get_width_format ()
+get_width_format (void)
 {
   static char buffer[256];
   int full_width;
@@ -250,9 +250,7 @@ get_width_format ()
   else
     {
       if (buffer[0] != '1'
-	  /* FIXME: assumes that decimal_point is a single character
-	     string.  */
-	  || buffer[1] != decimal_point[0]
+	  || buffer[1] != decimal_point
 	  || buffer[2 + strspn (&buffer[2], "0123456789")] != '\0')
 	return "%g";
       width1 -= 2;
@@ -265,9 +263,7 @@ get_width_format ()
   else
     {
       if (buffer[0] != '1'
-	  /* FIXME: assumes that decimal_point is a single character
-	     string.  */
-	  || buffer[1] != decimal_point[0]
+	  || buffer[1] != decimal_point
 	  || buffer[2 + strspn (&buffer[2], "0123456789")] != '\0')
 	return "%g";
       width2 -= 2;
@@ -310,21 +306,21 @@ main (int argc, char **argv)
 
   atexit (close_stdout);
 
-  equal_width = 0;
+  equal_width = false;
   separator = "\n";
   first = 1.0;
 
-  /* Figure out the locale's idea of a decimal point.  */
-#if HAVE_LOCALECONV
+  /* Get locale's representation of the decimal point.  */
   {
-    struct lconv *locale;
+    struct lconv const *locale = localeconv ();
 
-    locale = localeconv ();
-    /* Paranoia.  */
-    if (locale && locale->decimal_point && locale->decimal_point[0] != '\0')
-      decimal_point = locale->decimal_point;
+    /* If the locale doesn't define a decimal point, or if the decimal
+       point is multibyte, use the C locale's decimal point.  FIXME:
+       add support for multibyte decimal points.  */
+    decimal_point = locale->decimal_point[0];
+    if (! decimal_point || locale->decimal_point[1])
+      decimal_point = '.';
   }
-#endif
 
   /* We have to handle negative numbers in the command line but this
      conflicts with the command line arguments.  So explicitly check first
@@ -332,7 +328,7 @@ main (int argc, char **argv)
   while (optind < argc)
     {
       if (argv[optind][0] == '-'
-	  && ((optc = argv[optind][1]) == decimal_point[0]
+	  && ((optc = argv[optind][1]) == decimal_point
 	      || ISDIGIT (optc)))
 	{
 	  /* means negative number */
@@ -345,9 +341,6 @@ main (int argc, char **argv)
 
       switch (optc)
 	{
-	case 0:
-	  break;
-
 	case 'f':
 	  format_str = optarg;
 	  break;
@@ -357,7 +350,7 @@ main (int argc, char **argv)
 	  break;
 
 	case 'w':
-	  equal_width = 1;
+	  equal_width = true;
 	  break;
 
 	case_GETOPT_HELP_CHAR;
@@ -371,19 +364,19 @@ main (int argc, char **argv)
 
   if (argc - optind < 1)
     {
-      error (0, 0, _("too few arguments"));
+      error (0, 0, _("missing operand"));
       usage (EXIT_FAILURE);
     }
 
   if (3 < argc - optind)
     {
-      error (0, 0, _("too many arguments"));
+      error (0, 0, _("extra operand %s"), quote (argv[optind + 3]));
       usage (EXIT_FAILURE);
     }
 
   if (format_str && !valid_format (format_str))
     {
-      error (0, 0, _("invalid format string: `%s'"), format_str);
+      error (0, 0, _("invalid format string: %s"), quote (format_str));
       usage (EXIT_FAILURE);
     }
 
